/* ************************************************************************
 * Copyright 2006 Lucio Benfante
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *********************************************************************** */
package com.benfante.splasher;

import com.benfante.splasher.task.Task;
import com.benfante.splasher.task.TaskObserver;
import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.SplashScreen;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import javax.imageio.ImageIO;

/**
 * A class that observe tasks and draw a description and a progress bar on
 * a Java 6 splash screen.
 *
 * @author Lucio Benfante <a href="mailto:lucio@benfante.com">lucio@benfante.com</a>
 * @version $Id: SimpleSplashManager.java,v 1.3 2006-12-22 13:33:32 lucio Exp $
 */
public class SimpleSplashManager implements TaskObserver {
    private List<Task> tasks = new ArrayList<Task>();
    private int totalWeight = 0;
    private int completedWeigth = 0;
    private SplashScreen splash;
    private Graphics2D splashGraphics;
    private Color progressBarColor;
    private Color messageColor;
    private Font messageFont;
    
    /**
     * Creates a new instance of SimpleSplashManager
     */
    public SimpleSplashManager() {
        splash = SplashScreen.getSplashScreen();
        splashGraphics = splash.createGraphics();
    }
    
    /**
     * This method will be called when a task notify changes to the observer.
     *
     * @param task The task notifying the changes.
     */
    public void update(Task task) {
        // Reset splash image
        Dimension size = splash.getSize();
        splashGraphics.setComposite(AlphaComposite.Clear);
        splashGraphics.fillRect(0, 0, size.width, size.height);
        splashGraphics.setPaintMode();
        
        this.completedWeigth = 0;
        for (Task currTask : tasks) {
            this.completedWeigth += currTask.getCompleted()*currTask.getWeight()/100;
            if (currTask == task) {
                updateMessage(task.getDescription());
            }
        }
        updateProgressBar();
        splash.update();
    }
    
    /**
     * This method will be called when an observer is added to a task.
     *
     * @param task The task that is adding the observer.
     */
    public void notifyAdd(Task task) {
        tasks.add(task);
        totalWeight += task.getWeight();
    }
    
    /**
     * Update the message on the splash screen.
     *
     * @param msg The message
     */
    private void updateMessage(String msg) {
        Rectangle r = splash.getBounds();
        splashGraphics.setFont(this.messageFont);
        splashGraphics.setColor(this.messageColor);
        FontMetrics fm = splashGraphics.getFontMetrics();
        splashGraphics.drawString(msg, 1, (int)r.getHeight()-8-fm.getDescent());
    }
    
    /**
     * Update the progress bar on the splash screen.
     */
    private void updateProgressBar() {
        Rectangle r = splash.getBounds();
        splashGraphics.setColor(this.progressBarColor);
        double x = ((r.getWidth()-4)*this.completedWeigth)/this.totalWeight;
        splashGraphics.fill3DRect(2, (int)r.getHeight()-7, (int)x, 5, true);
    }
    
    /**
     * Set the progress bar color.
     *
     * @param progressBarColor The progress bar color
     */
    public void setProgressBarColor(Color progressBarColor) {
        this.progressBarColor = progressBarColor;
    }
    
    /**
     * Set the message color.
     *
     * @param messageColor The message color
     */
    public void setMessageColor(Color messageColor) {
        this.messageColor = messageColor;
    }
    
    /**
     * Set the message font.
     *
     *  @param messageFont The message font
     */
    public void setMessageFont(Font messageFont) {
        this.messageFont = messageFont;
    }
}
